write-host ("`nVelostrata - Azure Application User and Custom Role setup") -foreground "Green"

################## Login and Subscription ###################

# Sign-in with your Azure Account Credentials

Connect-AZAccount

# Select Azure Subscription

$subscriptionId = 
    (Get-AZSubscription |
     Out-GridView `
        -Title "Select an Azure Subscription ..." `
        -PassThru).Id

Select-AZSubscription `
    -SubscriptionId $subscriptionId

################## Variables ####################

$APPUserName = Read-Host "`nEnter a name for the Velostrata Application user"

write-host "`nCreating an Azure Application user for the Velostrata Operations in the subscription -" -nonewline; write-host " $SubscriptionId" -Foreground "Cyan"

$UniquePrefix = $APPUserName

$Secure_String_Pwd = ConvertTo-SecureString "VelosADApp123" -AsPlainText -Force

$VelosADApp = New-AZADApplication -DisplayName $APPUserName -HomePage "https://poc.$UniquePrefix" -IdentifierUris "https://poc.$UniquePrefix" -Password $Secure_String_Pwd

write-host

############################# Create Velostrata Operations Custom Role ##########################

write-host "`nCreating an Azure Custom Role for the Velostrata Operations in Subscription -" -nonewline; write-host " $SubscriptionId" -Foreground "Cyan"


$velosRoleName = "Velostrata Custom Role " + [guid]::NewGuid().ToString().Substring(0,2)

$jsonRoleTemplate = "{ `
    ""Name"":  ""$velosRoleName"", `
    ""Description"":  ""Velostrata Custom Role"", `
    ""Actions"":  [ `
                    ""Microsoft.Compute/availabilitySets/read"", `
					""Microsoft.Compute/availabilitySets/write"", `
					""Microsoft.Compute/availabilitySets/delete"", `
					""Microsoft.Compute/availabilitySets/vmSizes/read"", `
					""Microsoft.Compute/disks/read"", `
					""Microsoft.Compute/disks/write"", `
					""Microsoft.Compute/disks/delete"", `
					""Microsoft.Compute/locations/diskOperations/read"", `
					""Microsoft.Compute/locations/operations/read"", `
					""Microsoft.Compute/snapshots/read"", `
					""Microsoft.Compute/snapshots/write"", `
					""Microsoft.Compute/snapshots/delete"", `
					""Microsoft.Compute/virtualMachines/read"", `
					""Microsoft.Compute/virtualMachines/write"", `
					""Microsoft.Compute/virtualMachines/delete"", `
					""Microsoft.Compute/virtualMachines/start/action"", `
					""Microsoft.Compute/virtualMachines/powerOff/action"", `
					""Microsoft.Compute/virtualMachines/restart/action"", `
					""Microsoft.Compute/virtualMachines/deallocate/action"", `
					""Microsoft.Compute/virtualMachines/vmSizes/read"", `
					""Microsoft.Compute/virtualMachines/instanceView/read"", `
					""Microsoft.Compute/virtualMachines/extensions/read"", `
					""Microsoft.Compute/virtualMachines/extensions/write"", `
					""Microsoft.Compute/virtualMachines/extensions/delete"", `
					""Microsoft.Network/networkInterfaces/read"", `
					""Microsoft.Network/networkInterfaces/write"", `
					""Microsoft.Network/networkInterfaces/join/action"", `
					""Microsoft.Network/networkInterfaces/delete"", `
					""Microsoft.Network/publicIPAddresses/read"", `
					""Microsoft.Network/publicIPAddresses/write"", `
					""Microsoft.Network/publicIPAddresses/delete"", `
					""Microsoft.Network/publicIPAddresses/join/action"", `
					""Microsoft.Network/virtualNetworks/read"", `
					""Microsoft.Network/virtualNetworks/subnets/read"", `
					""Microsoft.Network/virtualNetworks/subnets/join/action"", `
					""Microsoft.Network/virtualNetworks/subnets/virtualMachines/read"", `
					""Microsoft.Network/virtualNetworks/virtualMachines/read"", `
					""Microsoft.Network/networkSecurityGroups/read"", `
					""Microsoft.Network/networkSecurityGroups/join/action"", `
					""Microsoft.Network/networkSecurityGroups/defaultSecurityRules/read"", `
					""Microsoft.Network/networkSecurityGroups/securityRules/read"", `
					""Microsoft.Storage/register/action"", `
					""Microsoft.Storage/checknameavailability/read"", `
					""Microsoft.Storage/storageAccounts/write"", `
					""Microsoft.Storage/storageAccounts/read"",`
					""Microsoft.Storage/storageAccounts/regeneratekey/action"",`
					""Microsoft.Storage/storageAccounts/delete"", `
					""Microsoft.Storage/storageAccounts/listkeys/action"", `
					""Microsoft.Storage/operations/read"",`
					""Microsoft.Resources/subscriptions/resourceGroups/read"", `
					""Microsoft.Resources/subscriptions/resourceGroups/write"", `
					""Microsoft.Resources/subscriptions/resourceGroups/delete"", `
					""Microsoft.Resources/subscriptions/resourcegroups/resources/read"", `
					""Microsoft.Resources/subscriptions/tagNames/read"", `
					""Microsoft.Resources/subscriptions/tagNames/write"", `
					""Microsoft.Resources/subscriptions/tagNames/delete"", `
					""Microsoft.Resources/subscriptions/tagNames/tagValues/read"", `
					""Microsoft.Resources/subscriptions/tagNames/tagValues/write"", `
					""Microsoft.Resources/subscriptions/tagNames/tagValues/delete"", `
					""Microsoft.Resources/subscriptions/resources/read"", `
					""Microsoft.Resources/subscriptions/read"", `
					""Microsoft.Resources/subscriptions/operationresults/read"", `
					""Microsoft.Resources/subscriptions/providers/read"", `
					""Microsoft.Resources/subscriptions/locations/read"", `
					""Microsoft.Compute/virtualMachines/convertToManagedDisks/action"", `
                ], `
    ""NotActions"":  [ `
                   ], `
    ""AssignableScopes"":  [ `
							""/subscriptions/SUBSCRIPTIONID"", `
                         ] `
}" 

$jsonRoleTemplate.replace("SUBSCRIPTIONID", $subscriptionId) | Out-File .\VelosRoleTemp.json	

New-AZRoleDefinition -InputFile .\VelosRoleTemp.json	

New-AZADServicePrincipal -ApplicationId $VelosADApp.ApplicationId

Write-Host "`Please wait - Assigning the Created Role to to Application user" -foreground "Yellow"

Start-Sleep -s 20

New-AZRoleAssignment -serviceprincipalname $VelosADApp.ApplicationId -RoleDefinitionName $VelosRoleName -scope "/subscriptions/$subscriptionId"

write-host "`nUser and Custom Role created successfully and assigned to subscription id -" -foreground "Yellow" -nonewline; write-host " $SubscriptionId" -Foreground "Cyan"
